MQTT_RC_SUCCESS = 0
MQTT_RC_NORMAL_DISCONNECTION = 0
MQTT_RC_GRANTED_QOS0 = 0
MQTT_RC_GRANTED_QOS1 = 1
MQTT_RC_GRANTED_QOS2 = 2
MQTT_RC_DISCONNECT_WITH_WILL_MSG = 4
MQTT_RC_NO_MATCHING_SUBSCRIBERS = 16
MQTT_RC_NO_SUBSCRIPTION_EXISTED = 17
MQTT_RC_CONTINUE_AUTHENTICATION = 24
MQTT_RC_REAUTHENTICATE = 25

MQTT_RC_UNSPECIFIED = 128
MQTT_RC_MALFORMED_PACKET = 129
MQTT_RC_PROTOCOL_ERROR = 130
MQTT_RC_IMPLEMENTATION_SPECIFIC = 131
MQTT_RC_UNSUPPORTED_PROTOCOL_VERSION = 132
MQTT_RC_CLIENTID_NOT_VALID = 133
MQTT_RC_BAD_USERNAME_OR_PASSWORD = 134
MQTT_RC_NOT_AUTHORIZED = 135
MQTT_RC_SERVER_UNAVAILABLE = 136
MQTT_RC_SERVER_BUSY = 137
MQTT_RC_BANNED = 138
MQTT_RC_SERVER_SHUTTING_DOWN = 139
MQTT_RC_BAD_AUTHENTICATION_METHOD = 140
MQTT_RC_KEEP_ALIVE_TIMEOUT = 141
MQTT_RC_SESSION_TAKEN_OVER = 142
MQTT_RC_TOPIC_FILTER_INVALID = 143
MQTT_RC_TOPIC_NAME_INVALID = 144
MQTT_RC_PACKET_ID_IN_USE = 145
MQTT_RC_PACKET_ID_NOT_FOUND = 146
MQTT_RC_RECEIVE_MAXIMUM_EXCEEDED = 147
MQTT_RC_TOPIC_ALIAS_INVALID = 148
MQTT_RC_PACKET_TOO_LARGE = 149
MQTT_RC_MESSAGE_RATE_TOO_HIGH = 150
MQTT_RC_QUOTA_EXCEEDED = 151
MQTT_RC_ADMINISTRATIVE_ACTION = 152
MQTT_RC_PAYLOAD_FORMAT_INVALID = 153
MQTT_RC_RETAIN_NOT_SUPPORTED = 154
MQTT_RC_QOS_NOT_SUPPORTED = 155
MQTT_RC_USE_ANOTHER_SERVER = 156
MQTT_RC_SERVER_MOVED = 157
MQTT_RC_SHARED_SUBS_NOT_SUPPORTED = 158
MQTT_RC_CONNECTION_RATE_EXCEEDED = 159
MQTT_RC_MAXIMUM_CONNECT_TIME = 160
MQTT_RC_SUBSCRIPTION_IDS_NOT_SUPPORTED = 161
MQTT_RC_WILDCARD_SUBS_NOT_SUPPORTED = 162
